import React, { useEffect, useState } from "react";
import {
  View,
  Platform,
  Text,
  TouchableOpacity,
  useWindowDimensions,
  LayoutAnimation,
} from "react-native";
import Constants from "expo-constants";
import { Ionicons } from "@expo/vector-icons";
import * as Updates from "expo-updates";

import { AS, ASCore } from "../storageControl";
import { news } from "../config/newsUpdate";
import { getStationList, lineList } from "../lib/getStationList";
import { injectJavascriptData } from "../lib/webViewInjectjavascript";
import { useCurrentTrain } from "../stateBox/useCurrentTrain";
import { useDeviceOrientationChange } from "../stateBox/useDeviceOrientationChange";
import { SheetManager } from "react-native-actions-sheet";
import TrainMenu from "../components/trainMenu";
import { EachTrainInfoCore } from "../components/ActionSheetComponents/EachTrainInfoCore";

import { useNavigation } from "@react-navigation/native";
import { useTrainMenu } from "../stateBox/useTrainMenu";
import { AppsWebView } from "./Apps/WebView";
import { NewMenu } from "./Apps/NewMenu";
import { MapsButton } from "./Apps/MapsButton";
import { ReloadButton } from "./Apps/ReloadButton";
import { LandscapeBackButton } from "./Apps/LandscapeBackButton";
import { useStationList } from "../stateBox/useStationList";
/* 
import StatusbarDetect from '../StatusbarDetect';
var Status = StatusbarDetect(); */

const top = Platform.OS == "ios" ? Constants.statusBarHeight : 0;

export default function Apps() {
  const { webview } = useCurrentTrain();
  const { height, width } = useWindowDimensions();
  const { navigate } = useNavigation();
  const { isLandscape } = useDeviceOrientationChange();
  const handleLayout = () => {};
  const { setInjectJavaScript, mapsStationData } = useTrainMenu();

  //画面表示関連
  const [iconSetting, setIconSetting] = useState(undefined);
  const [mapSwitch, setMapSwitch] = useState(undefined);
  const [stationMenu, setStationMenu] = useState(undefined);
  const [LoadError, setLoadError] = useState(false);

  //列車情報表示関連
  const [trainInfo, setTrainInfo] = useState({
    trainNum: undefined,
    limited: undefined,
    trainData: undefined,
  });

  //駅情報画面用
  const { originalStationList, setOriginalStationList } = useStationList();
  const [trainMenu, setTrainMenu] = useState("true");

  //地図表示テキスト
  const injectJavascript = injectJavascriptData(
    mapSwitch,
    iconSetting,
    stationMenu,
    trainMenu
  );

  useEffect(() => {
    //ニュース表示
    AS.getItem("status")
      .then((d) => {
        if (d != news) navigate("news");
      })
      .catch(() => navigate("news"));
  }, []);

  useEffect(() => {
    //列車アイコンスイッチ
    ASCore({ k: "iconSwitch", s: setIconSetting, d: "true", u: true });
    //地図スイッチ
    ASCore({ k: "mapSwitch", s: setMapSwitch, d: "false", u: true });
    //駅メニュースイッチ
    ASCore({ k: "stationSwitch", s: setStationMenu, d: "true", u: true });
    //列車メニュースイッチ
    ASCore({ k: "trainSwitch", s: setTrainMenu, d: "true", u: true });
  }, []);

  const openStationACFromEachTrainInfo = async (stationName) => {
    await SheetManager.hide("EachTrainInfo");
    const findStationEachLine = (selectLine) => {
      let NearStation = selectLine.filter((d) => d.Station_JP == stationName);
      return NearStation;
    };
    let returnDataBase = lineList
      .map((d) => findStationEachLine(originalStationList[d]))
      .filter((d) => d.length > 0)
      .reduce((pre, current) => {
        pre.push(...current);
        return pre;
      }, []);
    if (returnDataBase.length) {
      const payload = {
        currentStation: returnDataBase,
        navigate,
        goTo: "Apps",
        useShow: () => SheetManager.show("StationDetailView", { payload }),
        onExit: () => SheetManager.hide("StationDetailView"),
      };
      SheetManager.show("StationDetailView", { payload });
    } else {
      SheetManager.hide("StationDetailView");
    }
  };
  return (
    <View
      style={{
        height: "100%",
        paddingTop: top,
        flexDirection: isLandscape ? "row" : "column",
      }}
      onLayout={handleLayout}
    >
      {!trainInfo.trainNum && isLandscape ? (
        <TrainMenu
          style={{
            width: (width / 100) * 40,
            height: "100%",
            flexDirection: "column-reverse",
          }}
        />
      ) : null}
      {/* {Status} */}
      <AppsWebView
        {...{
          openStationACFromEachTrainInfo,
        }}
      />
      {isLandscape && trainInfo.trainNum && (
        <View
          style={{
            width: (width / 100) * 40,
            height: height,
            flexDirection: "column",
          }}
        >
          <EachTrainInfoCore
            {...{
              data: trainInfo.trainNum ? trainInfo : undefined,
              openStationACFromEachTrainInfo,
              from: "Train",
              setTrainInfo,
              navigate,
            }}
          />
        </View>
      )}
      {isLandscape || (
        <MapsButton
          onPress={() => {
            setInjectJavaScript("");
            navigate("trainMenu", { webview });
          }}
          mapSwitch={mapSwitch == "true" ? "flex" : "none"}
        />
      )}
      {isLandscape && trainInfo.trainNum && (
        <LandscapeBackButton
          onPress={() => {
            LayoutAnimation.easeInEaseOut();
            setTrainInfo({
              trainNum: undefined,
              limited: undefined,
              trainData: undefined,
            });
          }}
        />
      )}
      {mapSwitch == "true" ? (
        <ReloadButton
          onPress={() => Updates.reloadAsync()}
          right={isLandscape && trainInfo.trainNum ? (width / 100) * 40 : 0}
          LoadError={LoadError}
        />
      ) : (
        <NewMenu LoadError={LoadError} />
      )}
    </View>
  );
}
